@@ -19,10 +19,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'BINLIBDEST': '/usr/local/lib/python3.4',
  'BLDLIBRARY': '-L. -lpython3.4m',
  'BLDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ '
-              '-L/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+              '-L/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'BUILDEXE': '',
  'BUILDPYTHON': 'python',
- 'BUILD_GNU_TYPE': 'sparc64-unknown-openbsd5.9',
+ 'BUILD_GNU_TYPE': 'x86_64-unknown-openbsd6.1',
  'BYTESTR_DEPS': '\\',
  'CC': 'cc -pthread',
  'CCSHARED': '-fPIC',
@@ -37,9 +37,9 @@ build_time_vars = {'ABIFLAGS': 'm',
  'CONFIGURE_CPPFLAGS': '-I/usr/local/include',
  'CONFIGURE_LDFLAGS': '-L/usr/local/lib/',
  'CONFIG_ARGS': "'--with-ensurepip=no' '--enable-shared' "
-                "'--srcdir=/usr/obj/ports/Python-3.4.4/Python-3.4.4' "
+                "'--srcdir=/usr/obj/ports/Python-3.4.5/Python-3.4.5' "
                 "'--with-fpectl' '--with-threads' '--enable-ipv6' "
-                "'--with-cxx_main' '--with-system-expat' '--with-system-ffi' "
+                "'--with-system-expat' '--with-system-ffi' "
                 "'--prefix=/usr/local' '--sysconfdir=/etc' "
                 "'--mandir=/usr/local/man' '--infodir=/usr/local/info' "
                 "'--localstatedir=/var' '--disable-silent-rules' "
@@ -47,9 +47,9 @@ build_time_vars = {'ABIFLAGS': 'm',
                 "'LDFLAGS=-L/usr/local/lib/' 'CPPFLAGS=-I/usr/local/include'",
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.4m',
- 'COREPYTHONPATH': ':plat-openbsd5',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.4.4/Python-3.4.4/coverage.info',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.4.4/Python-3.4.4/lcov-report',
+ 'COREPYTHONPATH': ':plat-openbsd6',
+ 'COVERAGE_INFO': '/usr/obj/ports/Python-3.4.5/Python-3.4.5/coverage.info',
+ 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.4.5/Python-3.4.5/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
@@ -68,8 +68,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'DLINCLDIR': '.',
  'DLLLIBRARY': '',
  'DOUBLE_IS_ARM_MIXED_ENDIAN_IEEE754': 0,
- 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 1,
- 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_BIG_ENDIAN_IEEE754': 0,
+ 'DOUBLE_IS_LITTLE_ENDIAN_IEEE754': 1,
  'DYNLOADFILE': 'dynload_shlib.o',
  'ENABLE_IPV6': 1,
  'ENSUREPIP': 'no',
@@ -91,7 +91,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_ACOSH': 1,
  'HAVE_ADDRINFO': 1,
  'HAVE_ALARM': 1,
- 'HAVE_ALIGNED_REQUIRED': 1,
+ 'HAVE_ALIGNED_REQUIRED': 0,
  'HAVE_ALLOCA_H': 0,
  'HAVE_ALTZONE': 0,
  'HAVE_ASINH': 1,
@@ -177,8 +177,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HAVE_FUTIMESAT': 0,
  'HAVE_GAI_STRERROR': 1,
  'HAVE_GAMMA': 1,
- 'HAVE_GCC_ASM_FOR_X64': 0,
- 'HAVE_GCC_ASM_FOR_X87': 0,
+ 'HAVE_GCC_ASM_FOR_X64': 1,
+ 'HAVE_GCC_ASM_FOR_X87': 1,
  'HAVE_GCC_UINT128_T': 1,
  'HAVE_GETADDRINFO': 1,
  'HAVE_GETC_UNLOCKED': 1,
@@ -457,27 +457,27 @@ build_time_vars = {'ABIFLAGS': 'm',
  'HGBRANCH': '',
  'HGTAG': '',
  'HGVERSION': '',
- 'HOST_GNU_TYPE': 'sparc64-unknown-openbsd5.9',
+ 'HOST_GNU_TYPE': 'x86_64-unknown-openbsd6.1',
  'INCLDIRSTOMAKE': '/usr/local/include /usr/local/include '
                    '/usr/local/include/python3.4m '
                    '/usr/local/include/python3.4m',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.4m',
- 'INSTALL': '/usr/obj/ports/Python-3.4.4/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.4.4/bin/install -c  -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.4.4/bin/install -c -s  -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.4.4/bin/install -c  -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.4.4/bin/install -c -m 555',
+ 'INSTALL': '/usr/obj/ports/Python-3.4.5/bin/install -c',
+ 'INSTALL_DATA': '/usr/obj/ports/Python-3.4.5/bin/install -c  -m 644',
+ 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.4.5/bin/install -c -s  -m 755',
+ 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.4.5/bin/install -c  -m 755',
+ 'INSTALL_SHARED': '/usr/obj/ports/Python-3.4.5/bin/install -c -m 555',
  'INSTSONAME': 'libpython3.4m.so.1.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
  'LDCXXSHARED': 'c++ -pthread -shared -fPIC',
- 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+ 'LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'LDLAST': '',
  'LDLIBRARY': 'libpython3.4m.so',
  'LDLIBRARYDIR': '',
  'LDSHARED': 'cc -pthread -shared -fPIC -L/usr/local/lib/ '
-             '-L/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+             '-L/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'LDVERSION': '3.4m',
  'LIBC': '',
  'LIBDEST': '/usr/local/lib/python3.4',
@@ -493,19 +493,19 @@ build_time_vars = {'ABIFLAGS': 'm',
  'LIBRARY_OBJS_OMIT_FROZEN': '\\',
  'LIBS': '-lintl -lpthread  -lutil',
  'LIBSUBDIRS': 'tkinter tkinter/test tkinter/test/test_tkinter \\',
- 'LINKCC': 'c++ -pthread',
+ 'LINKCC': 'cc -pthread',
  'LINKFORSHARED': '-Wl,--export-dynamic',
  'LIPO_32BIT_FLAGS': '',
  'LN': 'ln',
  'LOCALMODLIBS': '',
  'LOG1P_DROPS_ZERO_SIGN': 0,
- 'MACHDEP': 'openbsd5',
- 'MACHDEPPATH': ':plat-openbsd5',
- 'MACHDEPS': 'plat-openbsd5',
+ 'MACHDEP': 'openbsd6',
+ 'MACHDEPPATH': ':plat-openbsd6',
+ 'MACHDEPS': 'plat-openbsd6',
  'MACHDEP_OBJS': '',
  'MACHDESTLIB': '/usr/local/lib/python3.4',
  'MACOSX_DEPLOYMENT_TARGET': '',
- 'MAINCC': 'c++ -pthread',
+ 'MAINCC': 'cc -pthread',
  'MAJOR_IN_MKDEV': 0,
  'MAJOR_IN_SYSMACROS': 0,
  'MAKESETUP': './Modules/makesetup',
@@ -547,7 +547,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PGENSRCS': '\\ \\',
  'PGOBJS': '\\',
  'PGSRCS': '\\',
- 'PLATDIR': 'plat-openbsd5',
+ 'PLATDIR': 'plat-openbsd6',
  'POBJS': '\\',
  'POSIX_SEMAPHORES_NOT_ENABLED': 0,
  'PROFILE_TASK': './Tools/pybench/pybench.py -n 2 --with-gc --with-syscheck',
@@ -561,7 +561,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PYTHONFRAMEWORKDIR': 'no-framework',
  'PYTHONFRAMEWORKINSTALLDIR': '',
  'PYTHONFRAMEWORKPREFIX': '',
- 'PYTHONPATH': ':plat-openbsd5',
+ 'PYTHONPATH': ':plat-openbsd6',
  'PYTHON_FOR_BUILD': './python -E',
  'PYTHON_HEADERS': '\\',
  'PYTHON_OBJS': '\\',
@@ -575,7 +575,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'PY_CPPFLAGS': '-I. -IInclude -I./Include -I/usr/local/include',
  'PY_FORMAT_LONG_LONG': '"ll"',
  'PY_FORMAT_SIZE_T': '"z"',
- 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+ 'PY_LDFLAGS': '-L/usr/local/lib/ -L/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'Py_DEBUG': 0,
  'Py_ENABLE_SHARED': 1,
  'Py_HASH_ALGORITHM': 0,
@@ -584,7 +584,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SGI_ABI': '',
@@ -626,10 +626,10 @@ build_time_vars = {'ABIFLAGS': 'm',
  'TCLTK_LIBS': '',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.4/Python-3.4.4 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.5/Python-3.4.5 '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.4/Python-3.4.4 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.4.5/Python-3.4.5 '
                './python ./Tools/scripts/run_tests.py',
  'TESTTIMEOUT': 3600,
  'THREADOBJ': 'Python/thread.o',
@@ -640,7 +640,7 @@ build_time_vars = {'ABIFLAGS': 'm',
  'UNIVERSALSDK': '',
  'USE_COMPUTED_GOTOS': 0,
  'USE_INLINE': 1,
- 'VA_LIST_IS_ARRAY': 0,
+ 'VA_LIST_IS_ARRAY': 1,
  'VERSION': '3.4',
  'WANT_SIGFPE_HANDLER': 1,
  'WINDOW_HAS_FLAGS': 1,
@@ -654,8 +654,8 @@ build_time_vars = {'ABIFLAGS': 'm',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.4.4/Python-3.4.4',
- 'abs_srcdir': '/usr/obj/ports/Python-3.4.4/Python-3.4.4',
+ 'abs_builddir': '/usr/obj/ports/Python-3.4.5/Python-3.4.5',
+ 'abs_srcdir': '/usr/obj/ports/Python-3.4.5/Python-3.4.5',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
