@@ -6,6 +6,7 @@ import getpass
 
 from distutils.command.install_headers import install_headers
 from distutils.tests import support
+from test.support import run_unittest
 
 class InstallHeadersTestCase(support.TempdirManager,
                              support.LoggingSilencer,
@@ -37,4 +38,4 @@ def test_suite():
     return unittest.makeSuite(InstallHeadersTestCase)
 
 if __name__ == "__main__":
-    unittest.main(defaultTest="test_suite")
+    run_unittest(test_suite())
